#
#   File        : pm_rpm.py
#   Author      : Gustavo Niemeyer <niemeyer@conectiva.com>
#   Description :
#
#   Copyright (C) 2003 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   * Thu Mar 20 2003 Alain TAUCH <alain.tauch@thalesatm.com>
#   - group/size/description added to package class + opendb change
#
#   * Tue Apr  8 2003 Alain TAUCH <alain.tauch@thalesatm.com>
#   - use standard rpm instead of rpm404                           
#
#   * Fri May 30 2003 Alain TAUCH <alain.tauch@thalesatm.com>
#   - summary added to package class                               
#
#   * Tue Aug 12 2003 Alain TAUCH <alain.tauch@thalesatm.com>
#   - fsize added to package class
#   - new flags added
#
#   * Wed Feb 15 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - next() call on match iterator no more supported (rpm-python 4.3.3)
#

from package_mgr import *
import rpm

__all__ = ["RPMPackageManager"]


# flags below are taken from genhdlist.c
# the first 2 flags are quite old (since rh6 I think)
# the last ones are recent (since rh8/9 ?)
FILENAME_TAG = 1000000
FILESIZE_TAG = 1000001
CDNUM_TAG    = 1000002
ORDER_TAG    = 1000003
MATCHER_TAG  = 1000004
#

class RPMPackageManager(PackageManager):
	"""Handles RPM packages."""

	type = "rpm"
	
	def who_provides(self,
					 pkgset,
					 name,
					 epoch=None,
					 version=None,
					 release=None,
					 flags=0,
					 provhash=None):
		ret = []
		if provhash == None:
			provhash = {}
			for pkg in pkgset:
				for rel in pkg.provides:
					try:
						provhash[rel.name].append(rel)
					except KeyError:
						provhash[rel.name] = [rel]
		goodrc = []
		if flags & PKG_LESS:
			goodrc.append(1)
		if flags & PKG_EQUAL:
			goodrc.append(0)
		if flags & PKG_GREATER:
			goodrc.append(-1)
		if not goodrc:
			goodrc = [-1, 0, 1]
		if provhash.has_key(name):
			for rel in provhash[name]:
				if version:
					if rel.version:
						if epoch:
							rc = cmp(epoch, rel.epoch)
						else:
							rc = 0
						if rc != 0:
							if rc in goodrc:
								ret.append(rel)
						else:
							rc = rpmvercmp(version, rel.version)
							if rc != 0:
								if rc in goodrc:
									ret.append(rel)
							else:
								if release != None:
									if rel.release != None:
										rc = rpmvercmp(release, rel.release)
										if rc in goodrc:
											ret.append(rel)
								elif rc in goodrc:
									ret.append(rel)
				else:
					ret.append(rel)
		return ret
	
	def who_requires(self,
					 pkgset,
					 name,
					 epoch=None,
					 version=None,
					 release=None,
					 reqhash=None):
		return self._who_relates(pkgset, name, epoch, version, release, reqhash,
								 "requires")
					
	def who_conflicts(self,
					  pkgset,
					  name,
					  epoch=None,
					  version=None,
					  release=None,
					  reqhash=None):
		return self._who_relates(pkgset, name, epoch, version, release, reqhash,
								 "conflicts")
					
	def who_obsoletes(self,
					  pkgset,
					  name,
					  epoch=None,
					  version=None,
					  release=None,
					  reqhash=None):
		return self._who_relates(pkgset, name, epoch, version, release, reqhash,
								 "obsoletes")

	def _who_relates(self,
					 pkgset,
					 name,
					 epoch,
					 version,
					 release,
					 relhash,
					 attrname):
		if relhash == None:
			relhash = {}
			for pkg in pkgset:
				for rel in getattr(pkg, attrname):
					try:
						relhash[rel.name].append(rel)
					except KeyError:
						relhash[rel.name] = [rel]
		ret = []
		if relhash.has_key(name):
			for rel in relhash[name]:
				goodrc = []
				if rel.flags & PKG_LESS:
					goodrc.append(1)
				if rel.flags & PKG_EQUAL:
					goodrc.append(0)
				if rel.flags & PKG_GREATER:
					goodrc.append(-1)
				if not goodrc:
					goodrc = [-1, 0, 1]
				if rel.name == name:
					if rel.version:
						if version:
							if epoch:
								rc = cmp(epoch, rel.epoch)
							else:
								rc = 0
							if rc != 0:
								if rc in goodrc:
									ret.append(rel)
							else:
								rc = rpmvercmp(rel.version, version)
								if rc != 0:
									if rc in goodrc:
										ret.append(rel)
								else:
									if rel.release != None:
										if release != None:
											rc = rpmvercmp(rel.release, release)
											if rc in goodrc:
												ret.append(rel)
									elif rc in goodrc:
										ret.append(rel)
					else:
						ret.append(rel)
		return ret


	def pkgset_from_hdlists(self, filenames, pkgset=None):
		"""Read given hdlists into given package set."""
		if not pkgset:
			pkgset = PackageSet(self)
		read_headers(pkgset, HeadersFromHdlists(filenames))
		return pkgset

	def pkgset_from_db(self, pkgset=None):
		"""Read installed database into given package set."""
		if not pkgset:
			pkgset = PackageSet(self)
		read_headers(pkgset, HeadersFromDB())
		return pkgset


def read_headers(pkgset, obj):
	requiredfiles = {}
	while 1:
		h = obj.next()
		if not h: break

		pkg = Package("rpm",
					  h[rpm.RPMTAG_NAME],
					  h[rpm.RPMTAG_EPOCH],
					  h[rpm.RPMTAG_VERSION],
					  h[rpm.RPMTAG_RELEASE],
					  h[rpm.RPMTAG_ARCH],
					  h[rpm.RPMTAG_GROUP],
					  h[rpm.RPMTAG_SIZE],
					  h[rpm.RPMTAG_DESCRIPTION],
					  h[rpm.RPMTAG_SUMMARY],
					  h[FILESIZE_TAG])
		pkgset.append(pkg)

		# Save header to process file dependencies latter. Will be removed.
		pkg.h = h
		 
		providename = h[rpm.RPMTAG_PROVIDENAME]
		provideflags = h[rpm.RPMTAG_PROVIDEFLAGS]
		provideversion = h[rpm.RPMTAG_PROVIDEVERSION]
		if type(provideflags) != ListType:
			provideflags = [provideflags]
		if providename:
			# List to avoid one package providing the same thing twice.
			# Will be removed.
			pkg.providelist = []
			for i in range(len(providename)):
				name = providename[i]
				epoch, version, release = parse_version(provideversion[i])
				flags = parse_flags(provideflags[i])
				flags |= PKG_PROVIDES
				tuple = (name, epoch, version, release, flags)
				if tuple not in pkg.providelist:
					pkg.providelist.append(tuple)
					rel = Relation(pkg, name, epoch, version, release, flags)
					pkg.provides.append(rel)

		requirename = h[rpm.RPMTAG_REQUIRENAME]
		requireflags = h[rpm.RPMTAG_REQUIREFLAGS]
		requireversion = h[rpm.RPMTAG_REQUIREVERSION]
		if type(requireflags) != ListType:
			requireflags = [requireflags]
		if requirename:
			for i in range(len(requirename)):
				name = requirename[i]
				if name[:6] == "rpmlib":
					continue
				if name[0] == "/":
					requiredfiles[name] = 1
				epoch, version, release = parse_version(requireversion[i])
				flags = parse_flags(requireflags[i])
				flags |= PKG_REQUIRES
				rel = Relation(pkg, name, epoch, version, release, flags)
				pkg.requires.append(rel)
	
		conflictname = h[rpm.RPMTAG_CONFLICTNAME]
		conflictflags = h[rpm.RPMTAG_CONFLICTFLAGS]
		conflictversion = h[rpm.RPMTAG_CONFLICTVERSION]
		if type(conflictflags) != ListType:
			conflictflags = [conflictflags]
		if conflictname:
			for i in range(len(conflictname)):
				name = conflictname[i]
				if name[:6] == "rpmlib":
					continue
				if name[0] == "/":
					conflictdfiles[name] = 1
				epoch, version, release = parse_version(conflictversion[i])
				flags = parse_flags(conflictflags[i])
				flags |= PKG_CONFLICTS
				rel = Relation(pkg, name, epoch, version, release, flags)
				pkg.conflicts.append(rel)
	
		obsoletename = h[rpm.RPMTAG_OBSOLETENAME]
		obsoleteflags = h[rpm.RPMTAG_OBSOLETEFLAGS]
		obsoleteversion = h[rpm.RPMTAG_OBSOLETEVERSION]
		if type(obsoleteflags) != ListType:
			obsoleteflags = [obsoleteflags]
		if obsoletename:
			for i in range(len(obsoletename)):
				name = obsoletename[i]
				if name[:6] == "rpmlib":
					continue
				if name[0] == "/":
					obsoletedfiles[name] = 1
				epoch, version, release = parse_version(obsoleteversion[i])
				flags = parse_flags(obsoleteflags[i])
				flags |= PKG_OBSOLETES
				rel = Relation(pkg, name, epoch, version, release, flags)
				pkg.obsoletes.append(rel)
	
	for pkg in pkgset:
		for filename in pkg.h[rpm.RPMTAG_FILENAMES]:
			if requiredfiles.has_key(filename):
				name = filename
				flags = PKG_PROVIDES|PKG_EXPLICIT
				tuple = (name, None, None, None, flags)
				if tuple not in pkg.providelist:
					pkg.providelist.append(tuple)
					rel = Relation(pkg, name, flags=flags)
					pkg.provides.append(rel)
		del pkg.h
		del pkg.providelist


class HeadersFromHdlists:
	"""
	Allow iteration over headers of given hdlists.
	"""
	
	def __init__(self, filenames):
		self._filenames = filenames
		self._fileindex = 0
		self._hdlist = None
		self._hdlistindex = 0
	
	def next(self):
		"""Returns the next header."""
		if self._hdlist == None or self._hdlistindex+1 == len(self._hdlist):
			if self._fileindex == len(self._filenames):
				return
			self._hdlist = rpm.readHeaderListFromFile( \
				self._filenames[self._fileindex])
			self._fileindex += 1
			self._hdlistindex = 0
		else:
			self._hdlistindex += 1
		return self._hdlist[self._hdlistindex]

class HeadersFromDB:
	"""
	Allow iteration over the installed database.
	"""
	
	def __init__(self):
		self._key = None
		self._db = None
		self._mi = None
		self._ts = None
		self._all = [];
		self._idx = 0;
	
	def next(self):
		"""Return next header"""
		if self._ts == None:
			self._ts = rpm.TransactionSet()
			self._mi = self._ts.dbMatch()
			# next() no more supported on _mi => store the result
			for i in self._mi:
				self._all.append(i)
		if self._idx >= len(self._all):
			return
		h = self._all[self._idx]
		self._idx += 1
		return h

def parse_version(string):
	epoch = None
	version = string
	release = None
	if version:
		pos = version.find(":")
		if pos != -1:
			epoch = version[:pos]
			version = version[pos+1:]
		pos = version.find("-")
		if pos != -1:
			release = version[pos+1:]
			version = version[:pos]
	else:
		version = None
	return epoch, version, release

def parse_flags(flags):
	ret = 0
	if flags & rpm.RPMSENSE_EQUAL:
		ret |= PKG_EQUAL
	if flags & rpm.RPMSENSE_LESS:
		ret |= PKG_LESS
	if flags & rpm.RPMSENSE_GREATER:
		ret |= PKG_GREATER
	if not (flags & rpm.RPMSENSE_FIND_PROVIDES) and \
	   not (flags & rpm.RPMSENSE_FIND_REQUIRES):
		ret |= PKG_EXPLICIT
	return ret

# compare alpha and numeric segments of two versions
# return 1: a is newer than b
#        0: a and b are the same version
#       -1: b is newer than a
def rpmvercmp(a, b):
	if a == b:
		return 0
	ai = 0
	bi = 0
	la = len(a)
	lb = len(b)
	while ai < la and bi < lb:
		while ai < la and not a[ai].isalnum(): ai += 1
		while bi < lb and not b[bi].isalnum(): bi += 1
		aj = ai
		bj = bi
		if a[aj].isdigit():
			while aj < la and a[aj].isdigit(): aj += 1
			while bj < lb and b[bj].isdigit(): bj += 1
			isnum = 1
		else:
			while aj < la and a[aj].isalpha(): aj += 1
			while bj < lb and b[bj].isalpha(): bj += 1
			isnum = 0
		if aj == ai or bj == bi:
			return -1
		if isnum:
			while ai < la and a[ai] == '0': ai += 1
			while bi < lb and b[bi] == '0': bi += 1
			if aj-ai > bj-bi: return 1
			if bj-bi > aj-ai: return -1
		rc = cmp(a[ai:aj], b[bi:bj])
		if rc:
			return rc
		ai = aj
		bi = bj
	if ai == la and bi == lb:
		return 0
	if ai == la:
		return -1
	else:
		return 1

# vim:ts=4:sw=4
