#
#   File        : winpref.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : the WinPref class
#
#   Copyright (C) 2003-2006 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   * Tue Feb 14 2006 Alain TAUCH <alain.tauch@thalesatm.com> 
#   - change deprecated gtk.Table.attach_defaults to gtk.Table.attach (pyGTK 2.0)
#   * Thu Jun 08 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - case insensitive sorting option
#   - delete_event mishandle fix
#   * Tue Jun 13 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - save/restore user preferences
#

import gtk
import gobject
import cPickle
import os

from gtk import TRUE, FALSE

class WinPrefException(Exception):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return (self.value)

 
class WinPref:
    """Preferences window for rpm-analyzer
    - Comps behaviour settings
    - hdlist import settings
    - Packages display settings
    """
    DUMMY   = 1
    GM_HASH = 1 << 0
    GD_HASH = 1 << 1
    GO_HASH = 1 << 2
    PM_HASH = 1 << 3
    PD_HASH = 1 << 4
    PO_HASH = 1 << 5
    CV_HASH = 1 << 0
    CR_HASH = 1 << 1
    CF_HASH = 1 << 2
    CI_HASH = 1 << 3

    ROW_REL = 0
    ROW_386 = 1
    ROW_586 = 2
    ROW_686 = 3
    ROW_ATH = 4
    ROW_ASK = 5
    ROW_NO  = 6
    
    ROW_MAX = 7
    COL_MAX = 4

    BEHAVE_RH9 = 1 << 0
    BEHAVE_ALL = 1 << 1
    BEHAVE_ASK = 1 << 2

    BACKUP_PREF_VER = 1
    
    def __init__(self, callback):
        self.callback = callback
        self.init_members()
        self.create_window()

    # set default values
    def init_members(self):
        # comps options
        # g(roup) m(andatory)
        # g(roup) d(efault)
        # g(roup) o(ptional)
        # p(ackage) m(andatory)
        # p(ackage) d(efault)
        # p(ackage) o(ptional)
        self.gm = TRUE
        self.gd = TRUE
        self.go = FALSE
        self.pm = TRUE
        self.pd = TRUE
        self.po = FALSE
        # if we changed the comps prefs
        # set/reset on apply/ok
        self.changed_comps = FALSE
        
        # hdlist options
        self.hd_order = (self.ROW_386, self.ROW_686,
                         self.ROW_REL, self.ROW_ASK)
        self.changed_hdlist = FALSE
        
        # pkg options:
        # c(olumn) v(ersion)
        # c(olumn) r(elease)
        # c(olumn) f(ile/rpm size)
        # c(olumn) i(installed size)
        self.cv = TRUE
        self.cr = FALSE
        self.cf = FALSE
        self.ci = FALSE
        # if we changed the pkg prefs
        # set/reset on apply/ok
        self.changed_pkg = FALSE

        # dep options
        self.dep_behave = self.BEHAVE_RH9
        self.changed_dep = FALSE

        # case sensitive sorting
	self.case_sensitive_sort = TRUE
        self.changed_case = FALSE

    def create_window(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # set window properties
        self.window.set_border_width(8)
        self.window.set_title("RPM Analyzer - Preferences")
        self.window.set_default_size(400, 200)
        self.window.set_modal(FALSE)
        # connect events
        self.window.connect("delete_event", self.e_hide)
        # content
        # VBox for all components 
        vbox_main = gtk.VBox()
        vbox_main.set_spacing(4)
        self.window.add(vbox_main)
        # comps frame
        frame_comps = gtk.Frame("Comps behaviour")
        table_comps = gtk.Table(6, 3, TRUE)
        table_comps.set_border_width(5)
        label2 = gtk.Label("<small><i>Select as dependencies groups flagged "
                           "as: </i></small>")
        label2.set_use_markup(TRUE)
        label2.set_alignment(0, 1)
        label3 = gtk.Label("<small><i>Select as dependencies packages "
                           "flagged as: </i></small>")
        label3.set_use_markup(TRUE)
        label3.set_alignment(0, 1)
        self.cb_gm = gtk.CheckButton()
        self.cb_gm.set_label("mandatory")
        self.cb_gd = gtk.CheckButton()
        self.cb_gd.set_label("default")
        self.cb_go = gtk.CheckButton()
        self.cb_go.set_label("optional")
        self.cb_pm = gtk.CheckButton()
        self.cb_pm.set_label("mandatory")
        self.cb_pd = gtk.CheckButton()
        self.cb_pd.set_label("default")
        self.cb_po = gtk.CheckButton()
        self.cb_po.set_label("optional")
        table_comps.attach(label2, 0, 2, 0, 1)
        table_comps.attach(self.cb_gm, 2, 3, 0, 1)
        table_comps.attach(self.cb_gd, 2, 3, 1, 2)
        table_comps.attach(self.cb_go, 2, 3, 2, 3)
        table_comps.attach(gtk.HSeparator(), 0, 3, 3, 4)
        table_comps.attach(label3, 0, 2, 4, 5)
        table_comps.attach(self.cb_pm, 2, 3, 4, 5)
        table_comps.attach(self.cb_pd, 2, 3, 5, 6)
        table_comps.attach(self.cb_po, 2, 3, 6, 7)
        frame_comps.add(table_comps)
        vbox_main.add(frame_comps)
        # hdlist frame
        frame_hdlist = gtk.Frame("Hdlist behaviour - import conflicts "
                                 "resolution order")
        table_hdlist = gtk.Table(self.COL_MAX + 1, self.ROW_MAX + 1, FALSE)
        table_hdlist.set_border_width(5)
        label_head = gtk.Label(" ")
        label_head.set_use_markup(TRUE)
        label_count = []
        for i in range(self.COL_MAX):
            label_count += [gtk.Label("<small>%s</small>" % (i+1, )), ]
            label_count[i].set_use_markup(TRUE)
            label_count[i].set_alignment(0, 1)
            label_count[i].set_padding(7, 0)
        label_rel = gtk.Label("latest release")
        label_rel.set_alignment(0, 1)
        label_386 = gtk.Label("arch i386")
        label_386.set_alignment(0, 1)
        label_586 = gtk.Label("arch i586")
        label_586.set_alignment(0, 1)
        label_686 = gtk.Label("arch i686")
        label_686.set_alignment(0, 1)
        label_ath = gtk.Label("arch athlon")
        label_ath.set_alignment(0, 1)
        label_ask = gtk.Label("ask me")
        label_ask.set_alignment(0, 1)
        label_no = gtk.Label("I don\'t care")
        label_no.set_alignment(0, 1)

        self.rb_hdlist = {}
        for i in range(self.COL_MAX):
            for j in range(self.ROW_MAX):
                if not j:
                    self.rb_hdlist[i] = [gtk.RadioButton(None), ]
                else:
                    self.rb_hdlist[i] += [gtk.RadioButton(self.rb_hdlist[i][0]),
                                          ]
        # here's how I count:
        # i -> gtk.Table row
        # j -> gtk.Table col
        # x -> rb_hdlist row
        # y -> rb_hdlist col
        for i in range(self.COL_MAX + 1):
            for j in range(self.ROW_MAX + 1):
                x = i - 1
                y = j - 1
                if not i:
                    if not j:
                        table_hdlist.attach(label_head, i, i+1, j, j+1)
                    elif y == self.ROW_REL:
                        table_hdlist.attach(label_rel, i, i+1, j, j+1)
                    elif y == self.ROW_386:
                        table_hdlist.attach(label_386, i, i+1, j, j+1)
                    elif y == self.ROW_586:
                        table_hdlist.attach(label_586, i, i+1, j, j+1)
                    elif y == self.ROW_686:
                        table_hdlist.attach(label_686, i, i+1, j, j+1)
                    elif y == self.ROW_ATH:
                        table_hdlist.attach(label_ath, i, i+1, j, j+1)
                    elif y == self.ROW_ASK:
                        table_hdlist.attach(label_ask, i, i+1, j, j+1)
                    elif y == self.ROW_NO:
                        table_hdlist.attach(label_no, i, i+1, j, j+1)
                else:
                    if not j:
                        table_hdlist.attach(label_count[x], i, i+1,
                                                     j, j+1)
                    else:
                        table_hdlist.attach(self.rb_hdlist[x][y],
                                                     i, i+1, j, j+1)

        frame_hdlist.add(table_hdlist)
        vbox_main.add(frame_hdlist)
        # pkg frame
        frame_pkg = gtk.Frame("Packages Zone - columns display")
        table_pkg = gtk.Table(2, 2, TRUE)
        table_pkg.set_border_width(5)
        self.cb_cv = gtk.CheckButton()
        self.cb_cv.set_label("version")
        self.cb_cr = gtk.CheckButton()
        self.cb_cr.set_label("release")
        self.cb_cf = gtk.CheckButton()
        self.cb_cf.set_label("file/rpm size (KB)")
        self.cb_ci = gtk.CheckButton()
        self.cb_ci.set_label("installed size (KB)")
        table_pkg.attach(self.cb_cv, 0, 1, 0, 1)
        table_pkg.attach(self.cb_cr, 0, 1, 1, 2)
        table_pkg.attach(self.cb_cf, 1, 2, 0, 1)
        table_pkg.attach(self.cb_ci, 1, 2, 1, 2)
        frame_pkg.add(table_pkg)
        vbox_main.add(frame_pkg)
        # dep frame
        frame_dep = gtk.Frame("Packages - dependencies behaviour")
        table_dep = gtk.Table(2,2, TRUE)
        table_dep.set_border_width(5)
        self.rb_rh9 = gtk.RadioButton(None, "RH9")
        self.rb_all = gtk.RadioButton(self.rb_rh9, "Include all")
        self.rb_ask = gtk.RadioButton(self.rb_rh9, "ask me")
        table_dep.attach(self.rb_rh9, 0, 1, 0, 1)
        table_dep.attach(self.rb_all, 1, 2, 0, 1)
        table_dep.attach(self.rb_ask, 0, 1, 1, 2)
        frame_dep.add(table_dep)
        vbox_main.add(frame_dep)
        # misc frame
        frame_misc = gtk.Frame("Miscellaneous options")
        self.cb_case = gtk.CheckButton()
        self.cb_case.set_label("Case sensitive sorting")
        frame_misc.add(self.cb_case)
        vbox_main.add(frame_misc)
        # bottom buttons -> HBox
        hbox_buttons = gtk.HBox()
        hbox_buttons.set_spacing(4)
        ok = gtk.Button(stock="gtk-ok")
        ok.connect("clicked", self.e_pressed_ok)
        hbox_buttons.add(ok)
        cancel = gtk.Button(stock="gtk-cancel")
        cancel.connect("clicked", self.e_pressed_cancel)
        hbox_buttons.add(cancel)
        apply = gtk.Button(stock="gtk-apply")
        apply.connect("clicked", self.e_pressed_apply)
        hbox_buttons.add(apply)
	# I want my custom 'defaults' button
	hbox_defaults = gtk.HBox()
	hbox_defaults.set_spacing(0)
	def_image = gtk.Image()
	def_image.set_from_stock("gtk-undo", gtk.ICON_SIZE_MENU)
	hbox_defaults.add(def_image)
	def_label = gtk.Label("_Load defaults")
        def_label.set_use_underline(TRUE)
	hbox_defaults.add(def_label)
        defaults = gtk.Button()
	defaults.add(hbox_defaults)
        defaults.connect("clicked", self.e_pressed_defaults)
        hbox_buttons.add(defaults)
        vbox_main.add(hbox_buttons)
        # warning text
        
    # show_pref is the only way to display the pref window
    def show_all(self):
        self.set_status()
        self.window.show_all()

    # called before show_all to set the appropriate values
    def set_status(self):
        # comps
        self.cb_gm.set_active(self.gm)
        self.cb_gd.set_active(self.gd)
        self.cb_go.set_active(self.go)
        self.cb_pm.set_active(self.pm)
        self.cb_pd.set_active(self.pd)
        self.cb_po.set_active(self.po)
        # hdlist
        for i in range(len(self.hd_order)):
            self.rb_hdlist[i][self.hd_order[i]].set_active(TRUE)
        # pkg
        self.cb_cv.set_active(self.cv)
        self.cb_cr.set_active(self.cr)
        self.cb_cf.set_active(self.cf)
        self.cb_ci.set_active(self.ci)
        # dep
        if self.dep_behave == self.BEHAVE_ALL:
            self.rb_all.set_active(TRUE)
        elif self.dep_behave == self.BEHAVE_RH9:
            self.rb_rh9.set_active(TRUE)
        elif self.dep_behave == self.BEHAVE_ASK:
            self.rb_ask.set_active(TRUE)
        # misc
        self.cb_case.set_active(self.case_sensitive_sort)

    def get_comps_hash(self):
        res = 0
        if self.gm:
            res += self.GM_HASH
        if self.gd:
            res += self.GD_HASH
        if self.go:
            res += self.GO_HASH
        if self.pm:
            res += self.PM_HASH
        if self.pd:
            res += self.PD_HASH
        if self.po:
            res += self.PO_HASH
        return res

    # get a _real_ algorithm, this sux
    def get_hdlist_hash(self):
        res = 0
        for i in range(len(self.hd_order)):
            res += self.hd_order[i] << i
        return res

    def get_pkg_hash(self):
        res = 0
        if self.cv:
            res += self.CV_HASH
        if self.cr:
            res += self.CR_HASH
        if self.cf:
            res += self.CF_HASH
        if self.ci:
            res += self.CI_HASH
        return res

    def get_status(self):
        # comps part
        status_comps = self.get_comps_hash()
        self.gm = self.cb_gm.get_active()
        self.gd = self.cb_gd.get_active()
        self.go = self.cb_go.get_active()
        self.pm = self.cb_pm.get_active()
        self.pd = self.cb_pd.get_active()
        self.po = self.cb_po.get_active()
        if status_comps != self.get_comps_hash():
            self.changed_comps = TRUE
        else:
            self.changed_comps = FALSE
        # hdlist part
        status_hdlist = self.get_hdlist_hash()
        self.hd_order = []
        for i in range(self.COL_MAX):
            for j in range(self.ROW_MAX):
                if self.rb_hdlist[i][j].get_active():
                    self.hd_order += [j, ]
        if status_hdlist != self.get_hdlist_hash():
            self.changed_hdlist = TRUE
        else:
            self.changed_hdlist = FALSE
        # pkg part
        status_pkg = self.get_pkg_hash()
        self.cv = self.cb_cv.get_active()
        self.cr = self.cb_cr.get_active()
        self.cf = self.cb_cf.get_active()
        self.ci = self.cb_ci.get_active()
        if status_comps != self.get_pkg_hash():
            self.changed_pkg = TRUE
        else:
            self.changed_pkg = FALSE
        # dep part
        status_dep = self.dep_behave
        if self.rb_all.get_active():
            self.dep_behave = self.BEHAVE_ALL
        elif self.rb_rh9.get_active():
             self.dep_behave = self.BEHAVE_RH9
        elif self.rb_ask.get_active():
            self.dep_behave = self.BEHAVE_ASK
        self.changed_dep = (self.dep_behave != status_dep)
        # misc part
        if self.case_sensitive_sort != self.cb_case.get_active():
            self.changed_case = TRUE
        self.case_sensitive_sort = self.cb_case.get_active()

    # save/restore prefs
    def save_to_file(self, filename):
        try:
            f = open(filename, 'w')
            pk = cPickle.Pickler(f, 1)

            # version
            pk.dump(self.BACKUP_PREF_VER)

            # comps options
            pk.dump(self.gm)
            pk.dump(self.gd)
            pk.dump(self.go)
            pk.dump(self.pm)
            pk.dump(self.pd)
            pk.dump(self.po)

            # hdlist options
            pk.dump(self.hd_order)
        
            # pkg options:
            pk.dump(self.cv)
            pk.dump(self.cr)
            pk.dump(self.cf)
            pk.dump(self.ci)

            # dep options
            pk.dump(self.dep_behave)

            # case sensitive sorting 
            pk.dump(self.case_sensitive_sort)

            f.close()
        except:
            try:
                f.close()
                os.unlink(filename)
            except:
                pass
            raise WinPrefException("Error while saving preferences")

    def restore_from_file(self, filename):
        try:
            f = open(filename, 'r')
            upk = cPickle.Unpickler(f)
            ver = upk.load()
            if ver == 1:
                self.restore_from_v1(upk)
            else:
                raise WinPrefException("Unsupported preferences version")
            f.close()
        except:
            try:
                f.close()
            except:
                pass
            raise WinPrefException("Error while loading preferences")

    def restore_from_v1(self, upk):
        # comps options
        self.gm = upk.load()
        self.gd = upk.load()
        self.go = upk.load()
        self.pm = upk.load()
        self.pd = upk.load()
        self.po = upk.load()

        # hdlist options
        self.hd_order = upk.load()

        # pkg options:
        self.cv = upk.load()
        self.cr = upk.load()
        self.cf = upk.load()
        self.ci = upk.load()

        # dep options
        self.dep_behave = upk.load()

        # case sensitive sorting 
        self.case_sensitive_sort = upk.load()
 
            
    # EVENTS
    # Window Manager behaviour
    def e_hide(self, widget, event):
        self.window.hide()
        # do not destroy this window
        return TRUE

    def e_pressed_ok(self, widget):
        self.get_status()
        self.e_hide(self.window, self.DUMMY)
        self.callback()

    def e_pressed_apply(self, widget):
        self.get_status()
        self.callback()

    def e_pressed_cancel(self, widget):
        self.e_hide(self.window, self.DUMMY)

    def e_pressed_defaults(self, widget):
        self.init_members()
        self.changed_comps = TRUE
	self.changed_hdlist = TRUE
	self.changed_pkg = TRUE
	self.changed_dep = TRUE
	self.changed_case = TRUE
	self.set_status()
        self.callback()
    #! EVENTS

